const path = require("path");
const multer = require("multer");
const UserModel = require("../models/Users");
const HelpModel = require("../models/Helps");
const SettingModel = require("../models/Settings");
const ReservationModel = require("../models/Reservations");
const SubscriptionModel = require("../models/Subscriptions");
const ClientModel = require("../models/Finalclients");
const { format } = require("date-fns");
const EmployeeModel = require("../models/Employees");

const sendMail = require("./mailController");

/* BUSINESS OWNER*/

const accountAssets = multer.diskStorage({
  destination: (req, file, cb) => {
    cb(null, path.join(__dirname, process.env.ASSET_PATH + "/users/"));
  },
  filename: (req, file, cb) => {
    cb(
      null,
      file.fieldname + "-" + Date.now() + path.extname(file.originalname)
    );
  },
});

const upload = multer({
  storage: accountAssets,
}).fields([
  { name: "fav_icon", maxCount: 1 },
  { name: "logo", maxCount: 1 },
  { name: "profile", maxCount: 1 },
]);

// Fetch All business owner records

exports.dashboard = async function (req, res) {
  let total_owners = await UserModel.countDocuments();
  let total_clients = await ClientModel.countDocuments();
  let total_bookings = await ReservationModel.countDocuments();
  let result = await SubscriptionModel.aggregate([
    {
      $addFields: {
        amount: { $toDouble: "$amount" }, // Convert the amount to a number
      },
    },
    {
      $group: {
        _id: null, // Grouping everything into one group
        total_revenue: { $sum: "$amount" }, // Sum the 'amount' field
      },
    },
  ]);

  return res.status(200).json({
    status: "true",
    data: {
      total_owners: total_owners,
      total_clients: total_clients,
      total_bookings: total_bookings,
      total_revenue: result.length > 0 ? result[0].total_revenue : 0,
    },
  });
};

exports.accounts = async function (req, res) {
  try {
    let { status } = req.params;
    let given_date = format(new Date(Date.now()), "yyyy-MM-dd");

    const records = await UserModel.find({ status: status });

    if (!records) {
      return res.status(404).json({
        status: "false",
        message: "Account not found",
      });
    }
    for (let index = 0; index < records.length; index++) {
      try {
        const employees = await EmployeeModel.find({
          business_owner: records[index]._id,
          $or: [{ deleted_at: null }, { deleted_at: { $gt: given_date } }],
        });

        const clients = await ClientModel.find({
          business_owner: records[index]._id,
        });

        records[index].employees = employees;
        records[index].finalclients = clients;
      } catch (err) {
        console.error(`Error fetching data for record index ${index}:`, err);
      }
    }
    console.log(records);
    return res.status(200).json({
      status: "true",
      data: records,
    });
  } catch (err) {
    return res.status(500).json({
      status: "false",
      message: "Database error",
      error: err.message,
    });
  }
};

// Fetch single business owner record

exports.getaccount = async function (req, res) {
  try {
    const { id } = req.params;

    const record = await UserModel.findById(id);

    if (!record) {
      return res.status(404).json({
        status: "false",
        message: "Account not found",
      });
    }
    return res.status(200).json({
      status: "true",
      data: record,
      employees: records,
      finalclients: finalclients,
    });
  } catch (err) {
    return res.status(500).json({
      status: "false",
      message: "Database error",
      error: err.message,
    });
  }
};

exports.accountCreate = function (req, res) {
  upload(req, res, async function (err) {
    if (err) {
      return res.status(500).json({
        status: "false",
        message: "File upload error",
        error: err.message,
      });
    }

    const { name, username, email, language } = req.body;

    if (!name || !username || !email || !language) {
      return res.status(400).json({
        status: "false",
        message: "Missing required fields",
      });
    }

    // Optionally assign file paths to the request body
    if (req.files.fav_icon) {
      req.body.fav_icon = req.files.fav_icon[0].filename;
    }
    if (req.files.logo) {
      req.body.logo = req.files.logo[0].filename;
    }
    if (req.files.profile) {
      req.body.profile = req.files.profile[0].filename;
    }
    req.body.company_name_display = req.body.company_name;
    req.body.company_name = req.body.company_name
      .replace(/\s+/g, "")
      .toLowerCase();
    // console.log(req.body);return;
    const newAccount = new UserModel(req.body);

    try {
      const accountdetails = await newAccount.save();
      let mailData = {};
      mailData.from = "appkodesproduct@gmail.com";
      mailData.to = "guru@mailinator.com";
      mailData.subject = "Welcome Mail";
      mailData.title = "Hello Prasath";
      mailData.message = "This is tesing mail";
      mailData.template = "";
      sendMail(mailData);
      return res.status(201).json({
        status: "true",
        user_id: accountdetails._id,
        username: accountdetails.username,
        address: accountdetails.address,
        name: accountdetails.name,
        age: accountdetails.age,
      });
    } catch (err) {
      return res.status(500).json({
        status: "false",
        message: "Database error",
        error: err.message,
      });
    }
  });
};

exports.accountEdit = function (req, res) {
  upload(req, res, async function (err) {
    if (err) {
      return res.status(500).json({
        status: "false",
        message: "File upload error",
        error: err.message,
      });
    }

    const accountId = req.body.id;

    // Check for missing ID
    if (!accountId) {
      return res.status(400).json({
        status: "false",
        message: "Missing account ID",
      });
    }

    // Update file fields if new files are uploaded
    if (req.files.fav_icon) {
      req.body.fav_icon = req.files.fav_icon[0].filename;
    }
    if (req.files.logo) {
      req.body.logo = req.files.logo[0].filename;
    }
    if (req.files.profile) {
      req.body.profile = req.files.profile[0].filename;
    }

    try {
      const updatedAccount = await UserModel.findByIdAndUpdate(
        accountId,
        req.body,
        { new: true } // Return the updated document
      );

      if (!updatedAccount) {
        return res.status(404).json({
          status: "false",
          message: "Account not found",
        });
      }

      return res.status(200).json({
        status: "true",
        message: "Account updated successfully",
        data: updatedAccount,
      });
    } catch (err) {
      return res.status(500).json({
        status: "false",
        message: "Database error",
        error: err.message,
      });
    }
  });
};
/* BUSINESS OWNER*/

/* Help*/

exports.helpCreate = async function (req, res) {
  const { title, description } = req.body;

  if (!title || !description) {
    return res.status(400).json({
      status: "false",
      message: "Missing required fields",
    });
  }

  req.body.created_at = Date.now();

  const newHelp = new HelpModel(req.body);

  try {
    const helpdetails = await newHelp.save();
    return res.status(201).json({
      status: "true",
      data: helpdetails,
    });
  } catch (err) {
    return res.status(500).json({
      status: "false",
      message: "Database error",
      error: err.message,
    });
  }
};

exports.helpEdit = async function (req, res) {
  const helpId = req.body.id;

  if (!helpId) {
    return res.status(400).json({
      status: "false",
      message: "Missing Help ID",
    });
  }

  try {
    const updatedHelp = await HelpModel.findByIdAndUpdate(
      helpId,
      req.body,
      { new: true } // Return the updated document
    );

    if (!updatedHelp) {
      return res.status(404).json({
        status: "false",
        message: "Account not found",
      });
    }

    return res.status(200).json({
      status: "true",
      message: "Help updated successfully",
      data: updatedAccount,
    });
  } catch (err) {
    return res.status(500).json({
      status: "false",
      message: "Database error",
      error: err.message,
    });
  }
};

exports.helps = async function (req, res) {
  try {
    const records = await HelpModel.find();
    if (!records) {
      return res.status(404).json({
        status: "false",
        message: "Help not found",
      });
    }

    return res.status(200).json({
      status: "true",
      data: records,
    });
  } catch (err) {
    return res.status(500).json({
      status: "false",
      message: "Database error",
      error: err.message,
    });
  }
};

exports.gethelp = async function (req, res) {
  try {
    const { id } = req.params;

    const record = await HelpModel.findById(id);

    if (!record) {
      return res.status(404).json({
        status: "false",
        message: "Help not found",
      });
    }
    return res.status(200).json({
      status: "true",
      data: record,
    });
  } catch (err) {
    return res.status(500).json({
      status: "false",
      message: "Database error",
      error: err.message,
    });
  }
};

/* Help*/

/* Reservation*/

exports.reservations = async function (req, res) {
  try {
    const records = await ReservationModel.find()
      .populate("business_owner")
      .populate("final_client")
      .populate("employee");
    if (!records) {
      return res.status(404).json({
        status: "false",
        message: "No Records Found",
      });
    }

    return res.status(200).json({
      status: "true",
      data: records,
    });
  } catch (err) {
    return res.status(500).json({
      status: "false",
      message: "Database error",
      error: err.message,
    });
  }
};

exports.getreservation = async function (req, res) {
  try {
    const { id } = req.params;

    const record = await ReservationModel.findById(id);

    if (!record) {
      return res.status(404).json({
        status: "false",
        message: "Reservation not found",
      });
    }
    return res.status(200).json({
      status: "true",
      data: record,
    });
  } catch (err) {
    return res.status(500).json({
      status: "false",
      message: "Database error",
      error: err.message,
    });
  }
};

/* Reservation*/

/* Transaction*/

exports.transactions = async function (req, res) {
  try {
    const records = await SubscriptionModel.find();
    if (!records) {
      return res.status(404).json({
        status: "false",
        message: "No Record Found",
      });
    }

    return res.status(200).json({
      status: "true",
      data: records,
    });
  } catch (err) {
    return res.status(500).json({
      status: "false",
      message: "Database error",
      error: err.message,
    });
  }
};

exports.gettransaction = async function (req, res) {
  try {
    const { id } = req.params;

    const record = await SubscriptionModel.findById(id);

    if (!record) {
      return res.status(404).json({
        status: "false",
        message: "Transaction not found",
      });
    }
    return res.status(200).json({
      status: "true",
      data: record,
    });
  } catch (err) {
    return res.status(500).json({
      status: "false",
      message: "Database error",
      error: err.message,
    });
  }
};

/* Transaction*/

exports.commonSettings = async function (req, res) {
  try {
    const record = await SettingModel.findOne();

    if (!record) {
      return res.status(404).json({
        status: "false",
        message: "Record Not Found",
      });
    }
    return res.status(200).json({
      status: "true",
      data: record,
    });
  } catch (err) {
    return res.status(500).json({
      status: "false",
      message: "Database error",
      error: err.message,
    });
  }
};

exports.commonUpdate = async function (req, res) {
  if (req.body.id) {
    const settingId = req.body.id;
    try {
      const updatedAccount = await SettingModel.findByIdAndUpdate(
        settingId,
        req.body,
        { new: true } // Return the updated document
      );

      if (!updatedAccount) {
        return res.status(404).json({
          status: "false",
          message: "Record Not Found",
        });
      }

      return res.status(200).json({
        status: "true",
        message: "Common Setting Updated Successfully",
        data: updatedAccount,
      });
    } catch (err) {
      return res.status(500).json({
        status: "false",
        message: "Database error",
        error: err.message,
      });
    }
  } else {
    const newSetting = new SettingModel(req.body);
    try {
      const settingdetails = await newSetting.save();
      return res.status(201).json({
        status: "true",
        message: "Settings updated successfully",
        data: settingdetails,
      });
    } catch (err) {
      return res.status(500).json({
        status: "false",
        message: "Database error",
        error: err.message,
      });
    }
  }
};
